#include "xil_types.h"
#include "xparameters.h"
#include "xiomodule_l.h"

#define GPO1_OUT XPAR_IOMODULE_SINGLE_BASEADDR+XGO_OUT_OFFSET
#define GPI1_IN  XPAR_IOMODULE_SINGLE_BASEADDR+XGI_IN_OFFSET

#include "xio.h"
#include "microblaze_sleep.h"

extern u32 GPO1_shadow;

volatile int x,y,z;

void delay(int value) {	MB_Sleep(value); }

int i2c_delay() {
	for (y=0;y<1;y++) {	z++; }
	return z;
}

#define ACK 0x00
#define NACK 0x80

u8 get_SDA() { return XIo_In32(GPI1_IN)  & 1; }

void set_SDA(u8 value)
{
	if (value) { GPO1_shadow |= 0x00000001;	}
	else       { GPO1_shadow &= 0xfffffffe;	}
	XIo_Out32(GPO1_OUT, GPO1_shadow);
}

void set_SCL(u8 value)
{
	if (value)	{ GPO1_shadow |= 0x00000002; }
	else        { GPO1_shadow &= 0xfffffffd; }
	XIo_Out32(GPO1_OUT, GPO1_shadow);

}


//....................................................................
// This function generates an I2C Start Condition
//....................................................................
void i2c_start(void)
{
	set_SDA(2);		//SDA_TRIS = 1;                   // ensure SDA & SCL are high
	set_SCL(1);		//SCL = 1;

	//SDA_TRIS = 0;                   // SDA = output
	set_SDA(0); //SDA = 0;                        // pull SDA low

	i2c_delay();

	//SCL = 0;                        // pull SCL low
	set_SCL(0);
	i2c_delay();
}


//....................................................................
// This function generates an I2C Stop Condition
//....................................................................
void i2c_stop(void)
{
	set_SCL(0); //SCL = 0;                        // ensure SCL is low
	//SDA_TRIS = 0;                   // SDA = output
	//SDA = 0;                        // SDA low
	set_SDA(0);
	i2c_delay();
	//SCL = 1;                        // pull SCL high
	set_SCL(1);

	//SDA_TRIS = 1;                   // allow SDA to be pulled high
	set_SDA(2);
	i2c_delay();

	//SCL=0;                          // ensure SCL is low
	set_SCL(0);
	i2c_delay();
}


//....................................................................
// Outputs a bit to the I2C bus
//....................................................................
void bit_out(unsigned char data)
{
	set_SCL(0); //SCL = 0;            // ensure SCL is low

	//SDA_TRIS=0;                     // configure SDA as an output
	//SDA= (data>>7);                 // output the MSB
	set_SDA((data>>7) & 1);


	i2c_delay();
	set_SCL(1); //SCL = 1;                        // pull SCL high to clock bit

	i2c_delay();
	set_SCL(0); //SCL = 0;                        // pull SCL low for next bit
}


//....................................................................
// Inputs a bit from the I2C bus
//....................................................................
void bit_in(unsigned char *data)
{
	set_SCL(0); //SCL = 0;                        // ensure SCL is low
	set_SDA(2); //SDA_TRIS = 1;                   // configure SDA as an input
	i2c_delay();
	set_SCL(1); //SCL = 1;                        // bring SCL high to begin transfer


	i2c_delay();
	//*data |= SDA;                   // input the received bit
	*data |= get_SDA();

	set_SCL(0); //SCL = 0;                        // bring SCL low again.
}


//....................................................................
// Writes a byte to the I2C bus
//....................................................................
unsigned char i2c_wr(unsigned char data)
{
unsigned char i;                // loop counter
unsigned char ack;              // ACK bit

	ack = 0;
	for (i = 0; i < 8; i++)         // loop through each bit
    	{
			bit_out(data);              // output bit
			data = data << 1;           // shift left for next bit
    }

	bit_in(&ack);                   // input ACK bit
	return ack;
}


//....................................................................
// Reads a byte from the I2C bus
//....................................................................
unsigned char i2c_rd(unsigned char ack)
{
unsigned char i;                // loop counter
unsigned char ret=0;            // return value

	for (i = 0; i < 8; i++)         // loop through each bit
    	{
			ret = ret << 1;             // shift left for next bit
			bit_in(&ret);               // input bit
    	}

	bit_out(ack);                   // output ACK/NAK bit
	return ret;
}


//.............................................................................
//          Polls the bus for ACK from device
//.............................................................................
void ack_poll (unsigned char control)
{
unsigned char result=1;

	while(result)
		{
			i2c_start();            // generate Restart condition
			result=i2c_wr(control); // send control byte (WRITE command)
			i2c_delay();
        }

	i2c_stop();                     // generate Stop condition
}

#define PLL_WR 0xE0
#define PLL_RD 0xE1

void I2C_ByteWrite(unsigned int addr, unsigned char data)
{
	i2c_start();              // generate Start COndition
	i2c_wr(PLL_WR);           // send WRITE command


	i2c_wr(addr & 0xff);     // address LSB
	i2c_wr(data);            // send data

	i2c_stop();              // initiate Stop Condition
	i2c_delay();
	ack_poll(PLL_WR);        // ACK polling
}

unsigned char I2C_ByteRead(unsigned int addr)
{
unsigned char data;

	i2c_start();                 // generate Start COndition
	i2c_wr(PLL_WR);           // send WRITE command
	i2c_wr(addr & 0xff);         //      address LSB

	i2c_start();                 // generate Start COndition
	i2c_wr(PLL_RD);           // send READ command

	data=i2c_rd(NACK);           // read data from buffer
	i2c_stop();                  // send Stop Condition

	return data;                 // return data from I2C buffer
}




